<cfcomponent displayname="Quick Site Builder" hint="This cfc handles some of the processing of the XML from Open Office word documents">

<cffunction name="parseTable" returntype="string" displayname="Parse the open office XML of a table">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsedHead = "" />
	<cfset var sParsedBody = "" />
	<cfset var i = "" />

<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="table:table-column">
			<!--- Column Support to be added --->
		</cfcase>
		<cfcase value="table:table-header-rows">
			<cfset sParsedHead = sParsedHead & parseTHead(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfcase value="table:table-row">
			<cfset sParsedBody = sParsedBody & parseRow(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsedBody = sParsedBody & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>

<cfset sParsed = "<table>" & sParsedHead & "<tbody>" & sParsedBody & "</tbody></table>" & request.cr />
<cfreturn sParsed>
</cffunction>

<cffunction name="parseRow" returntype="string" displayname="Parse the open office XML of a Table Row">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />
<cfset sParsed = sParsed & "<tr>" & request.cr />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="table:table-cell">
			<cfset sParsed = sParsed & parseCell(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfcase value="table:covered-table-cell">
			<!--- Covered by colspan. TODO: add support for rowspan.
			http://books.evc-cit.info/ch04.php#vert-span-example
			 --->
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</tr>" & request.cr />
<cfreturn sParsed>
</cffunction>

<cffunction name="parseCell" returntype="string" displayname="Parse the open office XML of a Table Cell">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />

<cfif listFind(StructKeyList(arguments.xmlStruct.xmlAttributes),"table:number-columns-spanned")>
	<cfset sParsed = sParsed & "<td colspan=""" & arguments.xmlStruct.xmlAttributes['table:number-columns-spanned'] & """/>" />
<cfelse>
	<cfset sParsed = sParsed & "<td>" />
</cfif>

<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="text:p">
			<cfif arraylen(arguments.xmlStruct.xmlChildren) eq 1>
				<cfset  sParsed = sParsed & application.parseWordXml.parseParagraph(
					xmlStruct=arguments.xmlStruct.xmlChildren[i],
					bUsePTag="false") />
			<cfelse>
				<cfset sParsed = sParsed & application.parseWordXml.parseParagraph(
					xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
			</cfif>
		</cfcase>
		<cfcase value="text:unordered-list">
			<cfset  sParsed = sParsed & application.parseWordXml.parseUl(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfcase value="text:ordered-list">
			<cfset  sParsed = sParsed & application.parseWordXml.parseOl(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfcase value="table:sub-table">
			<!--- TODO: This is NOT ideal. This should really be done with colspan.
				  NOT a sub table. --->
			<cfset  sParsed = sParsed & application.parseWordXml.parseTable(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</td>" & request.cr />
<cfreturn sParsed>
</cffunction>


<cffunction name="parseUl" returntype="string" displayname="Parse the open office XML of a Unordered list">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />

<cfset sParsed = sParsed & "<ul>" & request.cr />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="text:list-item">
			<cfset sParsed = sParsed & parseLi(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</ul>" & request.cr />
<cfreturn sParsed>
</cffunction>

<cffunction name="parseLi" returntype="string" displayname="Parse the open office XML of a list item">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />
	
<cfset sParsed = sParsed & "<li>" />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="text:p">
			<cfif arraylen(arguments.xmlStruct.xmlChildren) eq 1>
				<cfset  sParsed = sParsed & application.parseWordXml.parseParagraph(
					xmlStruct=arguments.xmlStruct.xmlChildren[i],
					bUsePTag="false") />
			<cfelse>
				<cfset sParsed = sParsed & application.parseWordXml.parseParagraph(
					xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
			</cfif>
		</cfcase>
		<cfcase value="text:unordered-list">
			<cfset  sParsed = sParsed & application.parseWordXml.parseUl(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfcase value="text:ordered-list">
			<cfset  sParsed = sParsed & application.parseWordXml.parseOl(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
		<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</li>" & request.cr />
<cfreturn sParsed>
</cffunction>

<cffunction name="parseOl" returntype="string" displayname="Parse the open office XML of a Ordered list">
<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />

<cfset sParsed = sParsed & "<ol>" & request.cr />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="text:list-item">
			<cfset sParsed = sParsed & parseLi(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</ol>" & request.cr />
<cfreturn sParsed>
</cffunction>


<cffunction name="parseParagraph" returntype="string" displayname="Parse the open office XML of a paragraph">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfargument name="bUsePTag" type="boolean" required="no" default="true" hint="Is a paragraph &lt;p&gt; tag neeed for this paragraph" />
	<cfset var sParsed = "" />
	<cfset var i = "" />

<cfif arguments.bUsePTag>
	<cfset sParsed = sParsed & "<p>" />
</cfif>
<cfset sParsed = sParsed & trim(arguments.xmlStruct.xmlText) />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="draw:image">
			<cfset sParsed = sParsed & "<strong>Image to be added here</strong>" />
		</cfcase>
		<cfcase value="text:tab-stop">
			<!--- TODO: Support for tab-stop --->
		</cfcase>
		<cfcase value="text:s">
			<!--- white space.
			http://books.evc-cit.info/ch03.php#whitespace-section --->
			<cfset sParsed = sParsed & " " />
		</cfcase>
		<cfcase value="text:line-break">
			<cfset sParsed = sParsed & "<br />" />
		</cfcase>
		<cfcase value="text:span">
			<cfset sParsed = sParsed & parseSpan(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfcase value="text:a">
			<cfset sParsed = sParsed & parseLink(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfif arguments.bUsePTag>
	<cfset sParsed = sParsed & "</p>" & request.cr />
</cfif>
<cfif trim(sParsed) eq "<p></p>">
	<cfset sParsed = "" />
</cfif>
<cfreturn sParsed>
</cffunction>

<cffunction name="parseSpan" returntype="string" displayname="Parse the open office XML of a span">
<cfargument name="xmlStruct" type="any" required="Yes">
<cfargument name="insideLink" type="boolean" required="No" default="False">
	<cfset var sParsed = "" />
	<cfset var i = "" />

	<cfif arguments.insideLink>
		<cfset sParsed = trim(arguments.xmlStruct.XmlText) />
	<cfelseif len(trim(arguments.xmlStruct.XmlText))>
		<cfset sParsed = "<strong>Span misplaced:" & arguments.xmlStruct.XmlText & "</strong>" />
	</cfif>

<cfreturn sParsed>
</cffunction>

<cffunction name="parseLink" returntype="string" displayname="Parse the open office XML of a link">
<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />

	
<cfset sParsed = sParsed & "<a href=""" & arguments.xmlStruct.XmlAttributes['xlink:href'] & """>" />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="text:span">
			<cfset sParsed = sParsed &
				parseSpan(xmlStruct=arguments.xmlStruct.xmlChildren[i],
							insideLink=true) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</a>" & request.cr />
<cfreturn sParsed>
</cffunction>


<cffunction name="parseTHead" returntype="string" displayname="Parse the open office XML of a table header">
	<cfargument name="xmlStruct" type="any" required="Yes">
	<cfset var sParsed = "" />
	<cfset var i = "" />

<cfset sParsed = sParsed & "<thead>" & request.cr />
<cfloop index="i" from="1" to="#arraylen(arguments.xmlStruct.xmlChildren)#">
	<cfswitch expression="#arguments.xmlStruct.xmlChildren[i].XmlName#">
		<cfcase value="table:table-column">
			<!--- Column Support to be added --->
		</cfcase>
		<cfcase value="table:table-row">
			<cfset sParsed = sParsed & parseRow(xmlStruct=arguments.xmlStruct.xmlChildren[i]) />
		</cfcase>
		<cfdefaultcase>
			<cfset sParsed = sParsed & "<strong>" & arguments.xmlStruct.xmlChildren[i].XmlName & "</strong> to be added" />
		</cfdefaultcase>
	</cfswitch>
</cfloop>
<cfset sParsed = sParsed & "</thead>" & request.cr />
<cfreturn sParsed>
</cffunction>
</cfcomponent>